#!/bin/bash -f

if [[ $EUID -ne 0 ]]; then
	echo "Run with sudo"
	exit 1
fi

echo running $0 at $(date)

PATH=/opt/banyan/local/sbin:$PATH

MASK=0x40000000
# MARK is the packet mark set by the proxy on outgoing traffic
MARK=0x40000000/0x40000000
# MARK_EB is the packet mark on proxied traffic sent to containers/pods attached to the bridge
MARK_EB=0x20000000/0x60000000

echo removing ebtables
ebtables -t nat -F BANYAN
ebtables -t nat -D PREROUTING -j BANYAN
ebtables -t nat -X BANYAN

ebtables -t filter -F BANYANFILTER
ebtables -t filter -D OUTPUT -j BANYANFILTER
ebtables -t filter -X BANYANFILTER

echo removing iptables rules
for chain in BANYAN BANYANtmp BANYANnext; do
	while true; do
		iptables -t nat -D PREROUTING -p tcp -m connmark \! --mark ${MARK} -j ${chain}
		if [ $? -ne 0 ]; then
			break
		fi
	done
	while true; do
		iptables -t nat -D OUTPUT -p tcp -m connmark \! --mark ${MARK} -j ${chain}
		if [ $? -ne 0 ]; then
			break
		fi
	done
	iptables -t nat -F ${chain}
	iptables -t nat -X ${chain}
done

while true; do
	iptables -t filter -D INPUT -p tcp -m set --match-set BANYANJAIL src -j DROP
	if [ $? -ne 0 ]; then
		break
	fi
done
while true; do
	iptables -t mangle -D POSTROUTING -p tcp -j BANYANPOST
	if [ $? -ne 0 ]; then
		break
	fi
done
iptables -t mangle -F BANYANPOST
iptables -t mangle -X BANYANPOST

while true; do
	iptables -t mangle -D PREROUTING -p tcp -j BANYAN -m comment --comment "Intercept outgoing traffic from containers and incoming traffic from the network"
	if [ $? -ne 0 ]; then
		break
	fi
done
while true; do
	iptables -t mangle -D OUTPUT -p tcp -j BANYAN -m comment --comment "Intercept outgoing traffic from local processes"
	if [ $? -ne 0 ]; then
		break
	fi
done
iptables -t mangle -F BANYAN
iptables -t mangle -X BANYAN

while true; do
	iptables -t raw -D OUTPUT -p tcp -j BANYAN
	if [ $? -ne 0 ]; then
		break
	fi
done
while true; do
	iptables -t raw -D PREROUTING -p tcp -j BANYAN
	if [ $? -ne 0 ]; then
		break
	fi
done
iptables -t raw -F BANYAN
iptables -t raw -X BANYAN

for chain in BANYAN BANYANtmp BANYANnext; do
	while true; do
		iptables -t filter -D INPUT -p tcp -m connmark \! --mark ${MARK} -j ${chain}
		if [ $? -ne 0 ]; then
			break
		fi
	done
	while true; do
		iptables -t filter -D FORWARD -p tcp -m connmark \! --mark ${MARK} -j ${chain}
		if [ $? -ne 0 ]; then
			break
		fi
	done
	while true; do
		iptables -t filter -D OUTPUT -p tcp -m connmark \! --mark ${MARK} -j ${chain}
		if [ $? -ne 0 ]; then
			break
		fi
	done
	iptables -t filter -F ${chain}
	iptables -t filter -X ${chain}
done
while true; do
	iptables -t filter -D INPUT -p tcp -m connmark \! --mark ${MARK} -j BANYAN_BREAKER
	if [ $? -ne 0 ]; then
		break
	fi
done
iptables -t filter -F BANYAN_BREAKER
iptables -t filter -X BANYAN_BREAKER

# policy routing rules check if packet should be routed to a local socket that may be using a non-local IP address.
echo removing routing table
ip rule del fwmark ${MARK} lookup 100
ip route flush table 100
# ip route del local 0.0.0.0/1 dev lo table 100

echo removing ipsets
ipset destroy BANYANJAIL
for s in banyan banyanservice banyancidr banyanservicenext banyancidrnext; do
	ipset destroy ${s}
done

exit 0
